import bpy
import serial
import time
import os
import math
                         

from bpy.props import EnumProperty
from bpy.types import Operator, Panel
from bpy.utils import register_class, unregister_class

ser = serial.Serial('COM22', 9600)
time.sleep(2) # wait for the serial connection to initialize
con="happy"
#--------------------------------------------------------------------------------------------------
class ModalTimerOperator(bpy.types.Operator):
    """Operator which runs its self from a timer"""
    bl_idname = "wm.modal_timer_operator"
    bl_label = "Modal Timer Operator"

    _timer = None

    def modal(self, context, event):
        if event.type in {'RIGHTMOUSE', 'ESC'}:
            self.cancel(context)
            return {'CANCELLED'}

        if event.type == 'TIMER':
            global con
            
            if bpy.context.scene.frame_current < bpy.context.scene.frame_end :
                
                bpy.context.scene.frame_set(bpy.context.scene.frame_current + 1)
                
                xt,yt,zt = bpy.context.object.matrix_world.to_translation()
                xr,yr,zr = bpy.context.object.matrix_world.to_euler()
                
                step = bpy.context.object.constraints["Follow Path"].offset_factor * 300
                
                
                while con != b'start\r\n':
                    ser.write(str(123.321).encode())
                    con = ser.readline()                
                con='0'
                print("started")

                time.sleep(0.1)
                ser.write(str(step).encode())
                con = ser.readline()
                while con != b'dones\r\n':
                    con = ser.readline()                
                con='0'
                print(step)
                
                time.sleep(0.1)
                ser.write(str(math.degrees(zr)).encode())
                
                con = ser.readline()
                while con != b'doner\r\n':
                    con = ser.readline()                
                con='0'
                print(math.degrees(zr))
                
                time.sleep(0.1)
                ser.write(str(zt).encode())
                
                con = ser.readline()
                while con != b'donez\r\n':
                    con = ser.readline()                
                con='0'
                print(zt)
                print("Data Send \n")
                
                
                
                con = ser.readline()
                while con != b'done\r\n':
                    con = ser.readline()                
                con='0'
                print("Data excuted \n")
                
            else:
                # change theme color, silly!
                color = context.preferences.themes[0].view_3d.space.gradients.high_gradient
                color.s = 1.0
                color.h += 0.01  


        return {'PASS_THROUGH'}

    def execute(self, context):
        wm = context.window_manager
        self._timer = wm.event_timer_add(0.1, window=context.window)
        wm.modal_handler_add(self)
        return {'RUNNING_MODAL'}

    def cancel(self, context):
        wm = context.window_manager
        wm.event_timer_remove(self._timer)

#-------------------------------------------------------------------------------------------------- 
class TEST_PT_panel(Panel):
    bl_idname = 'TEST_PT_panel'
    bl_label = 'Test'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Test'
 
    def draw(self, context):
        layout = self.layout
        layout.operator('test.test_op', text='Reset completely').action = 'HM'
        layout.operator('test.test_op', text='start streaming').action = 'RF'
        layout.operator('test.test_op', text='Disconnect serial').action = 'DC'
 
#--------------------------------------------------------------------------------------------------
class TEST_OT_test_op(Operator):
    bl_idname = 'test.test_op'
    bl_label = 'Test'
    bl_description = 'Test'
    bl_options = {'REGISTER', 'UNDO'}
 
    action: EnumProperty(
        items=[
            ('HM', 'Home', 'Home'),
            ('RF', 'Refresh', 'Refresh'),
            ('DC', 'Disconnect', 'Disconnect')
        ]
    )
 
    def execute(self, context):
        if self.action == 'HM':
            self.reset_scene(context=context)
        elif self.action == 'RF':
            self.start_trans(context=context)
        elif self.action == 'DC':
            self.disconnect_port(context=context)
        return {'FINISHED'}
 
    @staticmethod
    def reset_scene(context):
        print("1")
        # Define the serial port and baud rate.
        
        bpy.ops.object.select_all(action='SELECT')
        bpy.ops.object.delete(use_global=False, confirm=False)
        bpy.ops.mesh.primitive_cube_add(location=(0, 0, 0))

 
    @staticmethod
    def start_trans(context):
        print("sending...")
        bpy.ops.wm.modal_timer_operator()
        
 
    @staticmethod
    def disconnect_port(context):
        print("Port Disconnecting")
        ser.close()
 
 
 
 
def register():
    register_class(TEST_OT_test_op)
    register_class(TEST_PT_panel)
    bpy.utils.register_class(ModalTimerOperator)
 
 
def unregister():
    unregister_class(TEST_OT_test_op)
    unregister_class(TEST_PT_panel)
    bpy.utils.unregister_class(ModalTimerOperator)
 
 
if __name__ == '__main__':
    register()
    
    

